var Store = require('../../store');
var Promise = require('bluebird');

/*
 * MODEL
 */
exports.model = {
  /**
   * Executes the find
   *
   * @class Model
   * @method exec
   * @param {function} resolve - The resolve callback
   * @param {function} reject - The reject callback
   *
   * @callback
   * @param {array|object} result - Either a Collection of records or a single Record
   * @this Collection
   */
  exec: function(resolve, reject){
    var self = this.chain();
    var query = self.query;

    return self.promise(function(resolve, reject){
      var as_raw = self.getInternal('as_raw');

      self.callInterceptors('beforeFind', [query], function(okay){
        if(okay){        

          query.exec(function(err, resp) {
            
            self.logger.info(query.toString()); 
            
            if(err){
              return reject(new Store.SQLError(err));
            }
          
            var data = {
              sql: query.toString(),
              result: resp
            };
          

            
            self.logger.trace(resp);  
          
            if(as_raw){
              resolve(data.result);
            }else{
              self.callInterceptors('afterFind', [data], function(okay){
                if(okay){
                  resolve(data.result);              
                }else{
                  resolve(null);
                }
              }, reject);
            }          
            
          });   
        
        }else{
          resolve(null);
        }
      }, reject);
    }, resolve, reject);
  },
  
  
  
  toSql: function(){
    var sql;
    var query = this.query;
        
    //make async?
    this.callInterceptors('beforeFind', [query], function(){
      sql = query.toString();
    });
    
    return sql.replace(/`/g, '\"').replace(/'(\d+)'/g, '$1');
  }
  
};